#
# ContinuousTable.py
#
# Defines a mutable ordered table datatype using a
# continuous representation.

class Record:
    def __init__(self, k, v):
        self.key = k
        self.value = v
        
    def __str__(self):
        return "<" + str(self.key) + ", " + str(self.value) + ">"

class ContinuousTable:
    def __init__(self):
        self.items = []
          
    def insert(self, key, value):
        i = 0
        for i in range(len(self.items)):
            if self.items[i].key > key:
                self.items.insert (i, Record (key,value))
                return
        self.items.append (Record (key, value))
            
    def lookup(self, key):
        def lookuprange(items):
            if len(items) == 0: return None
            if len(items) == 1:
                if items[0].key == key:
                    return items[0].value
                else:
                    return None
            middle = len(items) / 2
            if key < items[middle].key:
                return lookuprange (items[:middle])
            else:
                return lookuprange (items[middle:])
                
        return lookuprange(self.items)
    
    def tlookup(self, key):
        def lookuprange(items):
            if len(items) == 0: return None
            if len(items) == 1:
                if items[0].key == key:
                    return items[0].value
                else:
                    return None
            split1 = len(items) / 3
            split2 = 2 * len(items) / 3
            
            if key < items[split1].key:
                return lookuprange (items[:split1])
            elif key < items[split2].key:
                return lookuprange (items[split1:split2])
            else:
                return lookuprange (items[split2:])
                
        return lookuprange(self.items)
        
    def length(self):
        return len(self.items)
    
    def __str__(self):
        res = "<"
        for index in range(len(self.items)):
            if not index == 0:
                res += ", "
            res += str(self.items[index])
        res += ">"
        return res